package com.pms.ocp.controller;

import com.pms.ocp.model.dto.GroupDto;
import com.pms.ocp.model.dto.OcpApiRequirementDto;
import com.pms.ocp.model.dto.TenantApiDto;
import com.pms.ocp.model.entity.ApiParam;
import com.pms.ocp.model.entity.OcpApiModelRel;
import com.pms.ocp.model.entity.OcpApiParam;
import com.pms.ocp.model.entity.OcpApiTenantRel;
import com.pms.ocp.model.vo.ResponseVO;
import com.pms.ocp.service.OcpApiRequirementService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * @author guofanfan
 * @date 2022/4/6
 */
@RequestMapping("/api-control")
@RestController
@Api(tags = "服务管控-新增-迭代 -下线")
public class ApiControlController {
    @Autowired
    private OcpApiRequirementService ocpApiRequirementService;

    /**
     * @param ocpApiRequirementDto
     * @return
     * @author guofanfan
     */
    @ApiOperation("新增-服务信息填写")
    @PostMapping("/addDemand")
    public ResponseVO addDemand(@RequestBody OcpApiRequirementDto ocpApiRequirementDto) {
        return ocpApiRequirementService.addDemand(ocpApiRequirementDto);
    }

    /**
     * @param ocpApiRequirementDto
     * @return
     * @author guofanfan
     */
    @ApiOperation("新增-关联模型")
    @PostMapping("/correlationModel")
    public ResponseVO correlationModel(@RequestBody OcpApiRequirementDto ocpApiRequirementDto) {
        return ocpApiRequirementService.correlationModel(ocpApiRequirementDto);
    }

    /**
     * @param ocpApiRequirementDto
     * @return
     * @author guofanfan
     */
    @ApiOperation("新增-关联应用")
    @PostMapping("/correlationTenant")
    public ResponseVO correlationTenant(@RequestBody OcpApiRequirementDto ocpApiRequirementDto) {
        return ocpApiRequirementService.correlationTenant(ocpApiRequirementDto);
    }

    /**
     * @param tenantGroupCompanyCode
     * @param tenantGroupCode
     * @return
     * @author guofanfan
     */
    @ApiOperation("新增-关联应用-左侧树")
    @GetMapping("/correlationTenantTree")
    public ResponseVO<List<GroupDto>> correlationTenantTree(@RequestParam(required = false) String tenantGroupCompanyCode,
                                                            @RequestParam(required = false) String tenantGroupCode) {
        List<GroupDto> list = ocpApiRequirementService.correlationTenantTree(tenantGroupCompanyCode, tenantGroupCode);
        return ResponseVO.ok(list);
    }

    /**
     * @param ocpApiRequirementDto
     * @return
     * @author guofanfan
     */
    @ApiOperation("新增-关联表字段")
    @PostMapping("/correlationTableField")
    public ResponseVO correlationTableField(@RequestBody OcpApiRequirementDto ocpApiRequirementDto) {
        return ocpApiRequirementService.correlationTableField(ocpApiRequirementDto);
    }


    /**
     * @param apiCode
     * @return
     * @Author：zhaochengming
     */
    @ApiOperation("服务--下线")
    @GetMapping("/updatetable")
    public ResponseVO updateTable(String apiCode) {
        ocpApiRequirementService.updateTables(apiCode);
        return ResponseVO.ok("请等待管理员审核");
    }

    /**
     * 迭代-模型回显
     * @param apiCode
     * @return
     * @author guofanfan
     */
    @ApiOperation("迭代-模型入参出参回显")
    @GetMapping("/echoModelParameter")
    public ResponseVO <List<OcpApiParam>> echoModelParameter(@RequestParam(required = false) String apiCode) {
        return  ocpApiRequirementService.echoModelParameter(apiCode);

    }

    @ApiOperation("迭代-模型回显")
    @GetMapping("/echoModel")
    public ResponseVO<List<OcpApiModelRel>> echoModel(@RequestParam(required = false) String apiCode) {
        return  ocpApiRequirementService.echoModel(apiCode);

    }

    /**
     * 迭代-表字段回显
     * @param apiCode
     * @return
     * @author guofanfan
     */
    @ApiOperation("迭代-表字段回显")
    @GetMapping("/echoTableField")
    public ResponseVO <Map<String,List<ApiParam>>>echoTableField(@RequestParam(required = false) String apiCode) {
        return ocpApiRequirementService.echoTableField(apiCode);
    }

    /**
     * @author guofanfan
     * @param apiCode
     * @return
     * 迭代-应用回显
     */
    @ApiOperation("迭代-应用回显")
    @GetMapping("/echoTenant")
    public ResponseVO <List<TenantApiDto>>echoTenant(@RequestParam(required = false) String apiCode) {
        return ocpApiRequirementService.echoTenant(apiCode);


    }

    @ApiOperation("迭代-服务信息填写")
    @PostMapping("/updateDemand")
    public ResponseVO updateDemand(@RequestBody OcpApiRequirementDto ocpApiRequirementDto) {
        return ocpApiRequirementService.updateDemand(ocpApiRequirementDto);
    }


    @ApiOperation("迭代-审核状态")
    @GetMapping("/selectAuditStatus")
    public ResponseVO selectAuditStatus(@RequestParam(required = false) String apiCode) {
        return ocpApiRequirementService.selectAuditStatus(apiCode);


    }

}
