package com.pms.ocp.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.pms.ocp.common.utils.SimilarityRatio;
import com.pms.ocp.common.utils.StringBusinessUtil;
import com.pms.ocp.mapper.AcceptInfoMapper;
import com.pms.ocp.mapper.OcpApiRequirementApprovalMapper;
import com.pms.ocp.model.dto.AcceptDTO;

import com.pms.ocp.model.dto.CommitDTO;
import com.pms.ocp.model.dto.RatioDTO;
import com.pms.ocp.model.entity.OcpApiRequirementAccept;

import com.pms.ocp.model.entity.OcpApiRequirementApproval;
import com.pms.ocp.model.vo.ApiAndModelVO;
import com.pms.ocp.model.vo.ResponseVO;
import com.pms.ocp.service.AcceptInfoService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.sql.Timestamp;
import java.util.List;

/**
 * 服务验收业务实现
 *
 * @author Administrator
 */
@Service
public class AcceptInfoServiceImpl implements AcceptInfoService {

    @Autowired
    AcceptInfoMapper acceptInfoMapper;

    @Autowired
    OcpApiRequirementApprovalMapper ocpApiRequirementApprovalMapper;

    /**
     * 验收通过
     */
    private static final String acceptOk = "1";

    /**
     * 验收不通过
     */
    private static final String acceptNo = "2";

    @Override
    public ResponseVO acceptInfo(AcceptDTO acceptDTO) {
        OcpApiRequirementAccept ocpApiRequirementAccept = new OcpApiRequirementAccept();
        if (acceptOk.equals(acceptDTO.getAcceptCode())) {
            ocpApiRequirementAccept = new OcpApiRequirementAccept();

            ocpApiRequirementAccept.setRequirementCode(acceptDTO.getRequirementCode());
            ocpApiRequirementAccept.setApiCode(acceptDTO.getApiCode());
            ocpApiRequirementAccept.setAcceptStats((long) 1);
            ocpApiRequirementAccept.setAcceptCompanyCode(acceptDTO.getAcceptCompanyCode());
            ocpApiRequirementAccept.setAcceptDepartment(acceptDTO.getAcceptDepartment());
            ocpApiRequirementAccept.setAcceptUserId(acceptDTO.getAcceptUserId());
            ocpApiRequirementAccept.setAcceptUserName(acceptDTO.getAcceptUserName());
            ocpApiRequirementAccept.setAcceptCtime(new Timestamp(System.currentTimeMillis()));
            ocpApiRequirementAccept.setIsDelete((long) 0);
        }
        if (acceptNo.equals(acceptDTO.getAcceptCode())) {
            ocpApiRequirementAccept = new OcpApiRequirementAccept();

            ocpApiRequirementAccept.setRequirementCode(acceptDTO.getRequirementCode());
            ocpApiRequirementAccept.setApiCode(acceptDTO.getApiCode());
            ocpApiRequirementAccept.setAcceptStats((long) 2);
            ocpApiRequirementAccept.setAcceptCompanyCode(acceptDTO.getAcceptCompanyCode());
            ocpApiRequirementAccept.setAcceptDepartment(acceptDTO.getAcceptDepartment());
            ocpApiRequirementAccept.setAcceptUserId(acceptDTO.getAcceptUserId());
            ocpApiRequirementAccept.setAcceptUserName(acceptDTO.getAcceptUserName());
            ocpApiRequirementAccept.setAcceptCtime(new Timestamp(System.currentTimeMillis()));
            ocpApiRequirementAccept.setIsDelete((long) 0);
        }

        LambdaQueryWrapper<OcpApiRequirementAccept> qw  =new LambdaQueryWrapper<>();
        int i = acceptInfoMapper.update(ocpApiRequirementAccept,qw.eq(OcpApiRequirementAccept::getRequirementCode,acceptDTO.getRequirementCode())) ;

        if (i > 0) {
            return ResponseVO.ok("验收成功");
        }
        return ResponseVO.error("");
    }

    /**
     * 重合度校验逻辑：1
     * 首先需要知道，重合度校验的思想是——文本校验——SimilarityRatio.getSimilarityRatio()。
     * <p>
     * 根据业务需求，新增的本条服务已经存入数据库中，故先要将这条存在数据库中的服务的is_delete状态置为已删除。
     * 后遍历排除掉本条服务之后的所有其他服务，用文本校验算法进行比对。
     * 算法得到的重合度值需要刷新入数据库中。
     * 查询排除掉本条服务的所有其余服务，按照重合度值排序取前五条。
     * 最后将先前is_delete置为已删除的状态恢复。
     * <p>
     * 模型重合度：三个字段。
     * 字段重合度：五个字段。
     *
     * @param ratioDTO
     * @return
     */
    @Override
    public ResponseVO apiModelRepeat(RatioDTO ratioDTO) {

        if (acceptInfoMapper.getNowApiCode(ratioDTO.getApiCode()) == 0) {
            return ResponseVO.error("当前服务尚未添加进数据库");
        }

        acceptInfoMapper.updateDel(ratioDTO.getApiCode());
        ApiAndModelVO vo = acceptInfoMapper.getRatioByApiCode(ratioDTO.getApiCode());
        // 模型重合度
        String str = vo.getApiCode() + vo.getiString() + vo.getoString();
        // 字段重合度
        String str2 = vo.getApiCode() + vo.getiString() + vo.getoString() + vo.getParam1() + vo.getParam2();


        List<ApiAndModelVO> list = acceptInfoMapper.getAllApiAndModel();

        if (StringBusinessUtil.isNullOrEmpty(list)) {
            return ResponseVO.error("所有服务数据仅一条,无法进行对比,请多增加服务数据以便对比");
        }

        for (ApiAndModelVO item : list) {

            // 模型重合度
            String tar = item.getApiCode() + item.getiString() + item.getoString();
            // 字段重合度
            String tar2 = item.getApiCode() + item.getiString() + item.getoString() + item.getParam1() + item.getParam2();

            // 模型重合度
            float ratio = SimilarityRatio.getSimilarityRatio(str, tar);
            // 字段重合度
            float ratio2 = SimilarityRatio.getSimilarityRatio(str2, tar2);

            // 模型重合度
            acceptInfoMapper.updateRatioByApiCode(ratio, item.getApiCode());
            // 字段重合度
            acceptInfoMapper.updateRatioParamByApiCode(ratio2, item.getApiCode());
        }
        List<ApiAndModelVO> finalList = acceptInfoMapper.getFinalRatio();

        acceptInfoMapper.returnDel(ratioDTO.getApiCode());
        return ResponseVO.ok(finalList);
    }

    @Override
    public ResponseVO apiModelRepeatCommit(CommitDTO commitDTO) {
        //先删
        if (StringUtils.isEmpty(commitDTO.getApiCode())){
            return ResponseVO.error("服务编码为空");
        }

        LambdaQueryWrapper<OcpApiRequirementApproval> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(StringUtils.isNotEmpty(commitDTO.getApiCode()),OcpApiRequirementApproval::getApiCode,commitDTO.getApiCode());
        ocpApiRequirementApprovalMapper.delete(wrapper);

        OcpApiRequirementApproval ocpApiRequirementApproval = new OcpApiRequirementApproval();
        ocpApiRequirementApproval.setRequirementCode(commitDTO.getRequirementCode());
        ocpApiRequirementApproval.setApprovalStats((long) 0);
        ocpApiRequirementApproval.setLastApprovalStats((long) 0);
        ocpApiRequirementApproval.setApprovalCompanyCode("0");
        ocpApiRequirementApproval.setApprovalDepartment("0");
        ocpApiRequirementApproval.setApprovalUserId("0");
        ocpApiRequirementApproval.setApprovalUserName("0");
        ocpApiRequirementApproval.setApprovalCtime(new Timestamp(System.currentTimeMillis()));
        ocpApiRequirementApproval.setIsDelete((long) 0);
        ocpApiRequirementApproval.setApiCode(commitDTO.getApiCode());

        if (ocpApiRequirementApprovalMapper.insert(ocpApiRequirementApproval) == 0) {
            return ResponseVO.error("提交失败");
        }
        return ResponseVO.ok("提交成功");
    }

    @Override
    public ResponseVO acceptPeopleInfo(AcceptDTO acceptDTO) {
        List<OcpApiRequirementApproval> list = acceptInfoMapper.getAcceptPeopleInfo(acceptDTO.getRequirementCode());

        return ResponseVO.ok(list);
    }
}
