package com.pms.ocp.service.impl;

import cn.afterturn.easypoi.cache.manager.IFileLoader;
import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.pms.ocp.mapper.*;
import com.pms.ocp.model.dto.ApiRequirementDto;
import com.pms.ocp.model.dto.GroupDto;
import com.pms.ocp.model.dto.OcpApiRequirementDto;
import com.pms.ocp.model.dto.TenantApiDto;
import com.pms.ocp.model.entity.*;
import com.pms.ocp.model.vo.ResponseVO;
import com.pms.ocp.service.OcpApiRequirementApprovalService;
import com.pms.ocp.service.OcpApiRequirementService;
import com.pms.ocp.service.ParamService;
import com.pms.ocp.service.TenantBasicManagementService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.sql.Timestamp;
import java.time.LocalDateTime;
import java.util.*;

@Service
public class OcpApiRequirementServiceImpl extends ServiceImpl<OcpApiRequirementMapper, OcpApiRequirement> implements OcpApiRequirementService {

    @Autowired
    private ApiBasicManagementMapper apiBasicManagementMapper;
    @Autowired
    private OcpApiRequirementMapper ocpApiRequirementMapper;

    @Autowired
    private OcpApiModelRelMapper ocpApiModelRelMapper;
    @Autowired
    private OcpApiTenantRelMapper ocpApiTenantRelMapper;
    @Autowired
    private OcpTenantGroupMapper ocpTenantGroupMapper;
    @Autowired
    private OcpTenantBaseMapper ocpTenantBaseMapper;
    @Autowired
    private OcpApiRequirementApprovalService ocpApiRequirementApprovalService;
    @Autowired
    private OcpApiRequirementApprovalMapper ocpApiRequirementApprovalMapper;

    @Autowired
    private ParamService paramService;

    @Autowired
    private ParamMapper paramMapper;
    @Autowired
    private OcpApiPMapper ocpApiPMapper;
    @Autowired
    private OcpApiExtentMapper ocpApiExtentMapper;
    @Autowired
    private OcpBaseMapper ocpBaseMapper;
    @Autowired
    private OcptenantRelMapper ocptenantRelMapper;
    @Autowired
    private OcpModelGroupMapper ocpModelGroupMapper;

    @Autowired
    private OcpApiBaseMapper ocpApiBaseMapper;

    @Autowired
    private OcpApiGroupMapper ocpApiGroupMapper;
    @Autowired
    private TenantBasicManagementService tenantBasicManagementService;

    /**
     * 关联表字段
     *
     * @param ocpApiRequirementDto
     * @return
     * @Author:郭凡凡
     */
    @Override
    public ResponseVO correlationTableField(OcpApiRequirementDto ocpApiRequirementDto) {
        if (StringUtils.isEmpty(ocpApiRequirementDto.getApiCode())) {
            return ResponseVO.error("服务编码为空");
        }
        //先删除 在添加
        LambdaQueryWrapper<ApiParam> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(StringUtils.isNotEmpty(ocpApiRequirementDto.getApiCode()), ApiParam::getApiCode, ocpApiRequirementDto.getApiCode());
        paramService.remove(wrapper);
        List<ApiParam> reqList = ocpApiRequirementDto.getReqList();
        List<ApiParam> respList = ocpApiRequirementDto.getRespList();
        bulkAddAndChildList(reqList, respList);
        ocpApiRequirementDto.setAddStatus("3");
        return ResponseVO.ok(ocpApiRequirementDto.getAddStatus());

    }

    /**
     * 批量添加包含子集
     *
     * @param
     */
    public void bulkAddAndChildList(List<ApiParam> reqList, List<ApiParam> respList) {
        reqList.stream().forEach(req -> {
            req.setProp("0");
            paramMapper.insert(req);
        });
        respList.stream().forEach(resp -> {
            resp.setProp("1");
            paramMapper.insert(resp);
        });
        for (ApiParam apiParam : reqList) {
            apiParam.setProp("0");
            bulkAddChild(apiParam);
        }
        for (ApiParam apiParam : respList) {
            apiParam.setProp("1");
            bulkAddChild(apiParam);
        }
    }

    /**
     * 递归添加子集
     *
     * @param apiParam
     */
    private void bulkAddChild(ApiParam apiParam) {
        String id = apiParam.getObjId();
        List<ApiParam> childList = apiParam.getChildrenList();
        if (childList != null) {
            childList.stream().forEach(apiParam1 -> {
                apiParam1.setPid(id);
                apiParam1.setProp(apiParam.getProp());
            });
            childList.stream().forEach(child -> paramMapper.insert(child));
            for (ApiParam childApiParam : childList) {
                bulkAddChild(childApiParam);
            }
        }
    }

    /**
     * @param ocpApiRequirementDto
     * @return
     * @Author:郭凡凡
     */
    @Override
    @Transactional
    public ResponseVO addDemand(OcpApiRequirementDto ocpApiRequirementDto) {
        if (StringUtils.isEmpty(ocpApiRequirementDto.getApiGroupCode())) {
            return ResponseVO.error("服务组编码不能为空");
        }
        if (StringUtils.isEmpty(ocpApiRequirementDto.getRequirementCode())) {
            return ResponseVO.error("需求编码不能为空");
        }
        if (ocpApiRequirementDto.getRequirementType() == null) {
            return ResponseVO.error("需求类型不能为空");
        }
        if (StringUtils.isEmpty(ocpApiRequirementDto.getApiName())) {
            return ResponseVO.error("服务名称不能为空");
        }
        if (StringUtils.isEmpty(ocpApiRequirementDto.getRequirementComments())) {
            return ResponseVO.error("需求简述不能为空");
        }
        if (StringUtils.isEmpty(ocpApiRequirementDto.getApiVersion())) {
            return ResponseVO.error("服务版本号不能为空");
        }
        if (StringUtils.isEmpty(ocpApiRequirementDto.getApiCode())) {
            return ResponseVO.error("服务编码不能为空");
        }
        //先删除  在添加
        //服务需求表

        LambdaQueryWrapper<OcpApiRequirement> wrapper1 = new LambdaQueryWrapper<>();
        wrapper1.eq(OcpApiRequirement::getApiCode, ocpApiRequirementDto.getApiCode());
        ocpApiRequirementMapper.delete(wrapper1);
        //服务基础表
        LambdaQueryWrapper<OcpApiBase> queryWrapper2 = new LambdaQueryWrapper<>();
        queryWrapper2.eq(OcpApiBase::getApiCode, ocpApiRequirementDto.getApiCode());
        apiBasicManagementMapper.delete(queryWrapper2);
        //服务扩展表
        LambdaQueryWrapper<OcpApiExtent> wrapper2 = new LambdaQueryWrapper<>();
        wrapper2.eq(OcpApiExtent::getApiCode, ocpApiRequirementDto.getApiCode());
        ocpApiExtentMapper.delete(wrapper2);

        //基础表
        OcpApiBase ocpApiBase = new OcpApiBase();
        ocpApiBase.setIsDelete(0);
        ocpApiBase.setApiCtime(Timestamp.valueOf(LocalDateTime.now()));
        ocpApiBase.setApiMtime(Timestamp.valueOf(LocalDateTime.now()));
        ocpApiBase.setApiCode(ocpApiRequirementDto.getApiCode());
        ocpApiBase.setApiName(ocpApiRequirementDto.getApiName());

        ocpApiBase.setApiGroupCode(ocpApiRequirementDto.getApiGroupCode());
        if (ocpApiRequirementDto.getApiPromotion() != null) {
            ocpApiBase.setApiPromotion(ocpApiRequirementDto.getApiPromotion());
        }
        if (StringUtils.isNotEmpty(ocpApiRequirementDto.getApiUnit())) {
            ocpApiBase.setApiUnit(ocpApiRequirementDto.getApiUnit());
        } else {
            ocpApiBase.setApiUnit("总部");
        }
        if (StringUtils.isNotEmpty(ocpApiRequirementDto.getApiUrl())) {
            ocpApiBase.setApiUrl(ocpApiRequirementDto.getApiUrl());
        } else {
            ocpApiBase.setApiUrl("/api-control/service");
        }

        //需求表
        OcpApiRequirement ocpApiRequirement = new OcpApiRequirement();
        ocpApiRequirement.setRequirementCode(ocpApiRequirementDto.getRequirementCode());
        ocpApiRequirement.setRequirementType(ocpApiRequirementDto.getRequirementType());
        ocpApiRequirement.setApiGroupCode(ocpApiRequirementDto.getApiGroupCode());
        ocpApiRequirement.setRequirementComments(ocpApiRequirementDto.getRequirementComments());
        ocpApiRequirement.setApiCode(ocpApiRequirementDto.getApiCode());
        ocpApiRequirement.setApiName(ocpApiRequirementDto.getApiName());
        ocpApiRequirement.setApiUnit(ocpApiRequirementDto.getApiUnit());
        ocpApiRequirement.setRequirementUserDepg(ocpApiRequirementDto.getRequirementUserDepg());
        if (ocpApiRequirementDto.getApiPromotion() != null) {
            ocpApiRequirement.setApiPromotion(ocpApiRequirementDto.getApiPromotion());
        }
        if (StringUtils.isNotEmpty(ocpApiRequirementDto.getApiVersion())) {
            ocpApiRequirement.setApiVersion(ocpApiRequirementDto.getApiVersion());
        }
        if (StringUtils.isNotEmpty(ocpApiRequirementDto.getApiUrl())) {
            ocpApiRequirement.setApiUrl(ocpApiRequirementDto.getApiUrl());
        } else {
            ocpApiRequirement.setApiUrl("/api-control/service");
        }
        if (StringUtils.isNotEmpty(ocpApiRequirementDto.getRequirementUserId())) {
            ocpApiRequirement.setRequirementUserName(ocpApiRequirementDto.getRequirementUserId());
        } else {
            ocpApiRequirement.setRequirementUserId("0");
        }
        if (StringUtils.isNotEmpty(ocpApiRequirementDto.getRequirementUserName())) {
            ocpApiRequirement.setRequirementUserName(ocpApiRequirementDto.getRequirementUserName());
        } else {
            ocpApiRequirement.setRequirementUserName("创建者");
        }
        ocpApiRequirement.setRequirementCtime(Timestamp.valueOf(LocalDateTime.now()));
        ocpApiRequirement.setRequirementMtime(Timestamp.valueOf(LocalDateTime.now()));
        ocpApiRequirement.setIsDelete((long) 0);


        //获取服务编码，名称 重复校验
        LambdaQueryWrapper<OcpApiBase> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotEmpty(ocpApiRequirementDto.getApiName()), OcpApiBase::getApiName, ocpApiRequirementDto.getApiName());
        queryWrapper.eq(OcpApiBase::getIsDelete, 0);
        List<OcpApiBase> ocpApiBases = apiBasicManagementMapper.selectList(queryWrapper);
        if (ocpApiBases.size() != 0) {
            return ResponseVO.error("服务名称已存在");
        }

        LambdaQueryWrapper<OcpApiBase> queryWrapper1 = new LambdaQueryWrapper<>();
        queryWrapper1.eq(StringUtils.isNotEmpty(ocpApiRequirementDto.getApiCode()), OcpApiBase::getApiCode, ocpApiRequirementDto.getApiCode());
        queryWrapper1.eq(OcpApiBase::getIsDelete, 0);
        List<OcpApiBase> ocpApiBases1 = apiBasicManagementMapper.selectList(queryWrapper1);
        if (ocpApiBases1.size() != 0) {
            return ResponseVO.error("服务编码已存在");
        }


        //也是重复校验 //需求表
        LambdaQueryWrapper<OcpApiRequirement> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(StringUtils.isNotEmpty(ocpApiRequirementDto.getRequirementCode()), OcpApiRequirement::getRequirementCode, ocpApiRequirementDto.getRequirementCode());
        List<OcpApiRequirement> ocpApiRequirements = ocpApiRequirementMapper.selectList(wrapper);
        if (ocpApiRequirements.size() != 0) {
            return ResponseVO.error("需求编码重复");
        }

        //添加到服务需求表 和 服务基础信息表 中
        //添加到服务扩展表中
        OcpApiExtent ocpApiExtent = new OcpApiExtent();
        ocpApiExtent.setApiCode(ocpApiRequirementDto.getApiCode());
        ocpApiExtent.setIsDelete(0);
        ocpApiExtent.setApiCtime(Timestamp.valueOf(LocalDateTime.now()));
        ocpApiExtent.setApiMtime(Timestamp.valueOf(LocalDateTime.now()));
        if (StringUtils.isNotEmpty(ocpApiRequirementDto.getClusterName())) {
            ocpApiExtent.setClusterName(ocpApiRequirementDto.getClusterName());
        } else {
            ocpApiExtent.setClusterName("测试集群");
        }
        if (StringUtils.isNotEmpty(ocpApiRequirementDto.getSpaceName())) {
            ocpApiExtent.setSpaceName(ocpApiRequirementDto.getSpaceName());
        } else {
            ocpApiExtent.setSpaceName("开发者");
        }
        int insert1 = ocpApiExtentMapper.insert(ocpApiExtent);

        int insert = apiBasicManagementMapper.insert(ocpApiBase);
        int insert2 = ocpApiRequirementMapper.insert(ocpApiRequirement);
        if ((insert2 > 0) && (insert > 0) && (insert1 > 0)) {
            ocpApiRequirementDto.setAddStatus("1");
            return ResponseVO.ok(ocpApiRequirementDto.getAddStatus());
        }
        return ResponseVO.error("添加失败");
    }

    /**
     * @param ocpApiRequirementDto
     * @return
     * @Author:郭凡凡 新增-关联模型
     */
    @Override
    public ResponseVO
    correlationModel(OcpApiRequirementDto ocpApiRequirementDto) {
        if (StringUtils.isEmpty(ocpApiRequirementDto.getApiCode())) {
            return ResponseVO.error("服务编码为空");
        }
        if (ocpApiRequirementDto.getModelCodes() == null) {
            return ResponseVO.error("请选择你要关联的模型");
        }
        List<String> list = Arrays.asList(ocpApiRequirementDto.getModelCodes());
        //先删除和其关联的模型
        LambdaQueryWrapper<OcpApiModelRel> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(StringUtils.isNotEmpty(ocpApiRequirementDto.getApiCode()), OcpApiModelRel::getApiCode, ocpApiRequirementDto.getApiCode());
        ocpApiModelRelMapper.delete(wrapper);


        //删除模型入参出参
        LambdaQueryWrapper<OcpApiParam> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotEmpty(ocpApiRequirementDto.getApiCode()), OcpApiParam::getApiCode, ocpApiRequirementDto.getApiCode());
        ocpApiPMapper.delete(queryWrapper);


        //然后添加关联关系

        for (String s : list) {
            OcpApiModelRel ocpApiModelRel = new OcpApiModelRel();
            ocpApiModelRel.setApiCode(ocpApiRequirementDto.getApiCode());
            ocpApiModelRel.setModelCode(s);
            ocpApiModelRel.setIsDelete(0);
            ocpApiModelRel.setRelationCtime(Timestamp.valueOf(LocalDateTime.now()));
            ocpApiModelRelMapper.insert(ocpApiModelRel);
        }
        List<OcpApiParam> ocpApiParamList = ocpApiRequirementDto.getOcpApiParamList();
        if (ocpApiParamList == null) {
            ocpApiRequirementDto.setAddStatus("2");
            return ResponseVO.ok(ocpApiRequirementDto.getAddStatus());
        }
        for (OcpApiParam ocpApiParam : ocpApiParamList) {
            if (StringUtils.isEmpty(ocpApiParam.getApiParamVersion())) {
                return ResponseVO.error("版本号为空");
            }
            ocpApiParam.setApiCode(ocpApiRequirementDto.getApiCode());
            ocpApiParam.setIsDelete(0);
            ocpApiPMapper.insert(ocpApiParam);
        }
        ocpApiRequirementDto.setAddStatus("2");
        return ResponseVO.ok(ocpApiRequirementDto.getAddStatus());
    }

    /**
     * 新增-关联应用
     *
     * @param ocpApiRequirementDto
     * @return
     * @Author:郭凡凡
     */
    @Override
    public ResponseVO correlationTenant(OcpApiRequirementDto ocpApiRequirementDto) {
        if (StringUtils.isEmpty(ocpApiRequirementDto.getApiCode())) {
            return ResponseVO.error("服务编码不能为空");
        }
        if (StringUtils.isEmpty(ocpApiRequirementDto.getApiTenantDescription())) {
            return ResponseVO.error("描述不能为空");
        }
        if (ocpApiRequirementDto.getTenantCodes() == null) {
            return ResponseVO.error("请选择你要关联的应用");
        }

        String[] tenantCodes = ocpApiRequirementDto.getTenantCodes();
        List<String> list = Arrays.asList(tenantCodes);
        //先删除和其关联的应用
        LambdaQueryWrapper<OcpApiTenantRel> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(StringUtils.isNotEmpty(ocpApiRequirementDto.getApiCode()), OcpApiTenantRel::getApiCode, ocpApiRequirementDto.getApiCode());
        ocpApiTenantRelMapper.delete(wrapper);
        //然后添加关系
        for (String s : list) {
            OcpApiTenantRel ocpApiTenantRel = new OcpApiTenantRel();
            ocpApiTenantRel.setApiCode(ocpApiRequirementDto.getApiCode());
            ocpApiTenantRel.setTenantCode(s);

            ocpApiTenantRel.setApiTenantDescription(ocpApiRequirementDto.getApiTenantDescription());
            ocpApiTenantRel.setApiTenantRelCtime(Timestamp.valueOf(LocalDateTime.now()));
            ocpApiTenantRel.setApiTenantRelMtime(Timestamp.valueOf(LocalDateTime.now()));
            ocpApiTenantRelMapper.insert(ocpApiTenantRel);
        }
        ocpApiRequirementDto.setAddStatus("4");
        return ResponseVO.ok(ocpApiRequirementDto.getAddStatus());


    }

    /**
     * 新增-关联应用-左侧树
     *
     * @param tenantGroupCompanyCode
     * @param tenantGroupCode
     * @return
     * @Author:郭凡凡
     */
    @Override
    public List<GroupDto> correlationTenantTree(String tenantGroupCompanyCode, String tenantGroupCode) {
        List<GroupDto> list = new ArrayList<>();
        if (StringUtils.isBlank(tenantGroupCode)) {
            List<GroupDto> ocpTenantGroups = ocpTenantGroupMapper.correlationTenantTree(tenantGroupCompanyCode, tenantGroupCode);
            for (GroupDto ocpTenantGroup : ocpTenantGroups) {
                if (StringUtils.isNotBlank(ocpTenantGroup.getTenantGroupCode()) && StringUtils.isNotBlank(ocpTenantGroup.getTenantGroupName())) {
                    list.add(ocpTenantGroup);
                }
            }
            return list;
        }
        List<GroupDto> baseList = ocpTenantBaseMapper.correlationTenantTree(tenantGroupCompanyCode, tenantGroupCode);
        return baseList;
    }

    /**
     * @param apiCode
     * @return
     * @Author:zhaochengming 查询回显--111
     */
    @Override
    public ApiRequirementDto getSelects(String apiCode) {

        //出参
        List<OcpApiParam> list = new ArrayList<>();
        //入参
        List<OcpApiParam> lists = new ArrayList<>();
        //应用描述
        List<String> listTenant = new ArrayList<>();
        //影响应用
        List<String> list1Tenat = new ArrayList<>();
        //影响模型
        List<Model> listModel = new ArrayList<>();
        //影响描述
        List<String> list1Model = new ArrayList<>();

        //返回值对象
        ApiRequirementDto apiRequirementDto = new ApiRequirementDto();
        //查询服务审批详情
        OcpApiBase ocpApiBase = ocpBaseMapper.selectCode(apiCode);
//        OcpApiBase ocpApiBase = ocpApiBaseMapper.selectById(objId);
        OcpApiRequirement ocpApiRequirement = ocpApiRequirementMapper.selectByCodes(ocpApiBase.getApiCode());

        //据apicode查询模型关系表
        List<OcpApiModelRel> model = ocpApiModelRelMapper.selectApiGroupds(ocpApiRequirement.getApiCode());
        //根据modelCode查询模型名称
        for (OcpApiModelRel ocpApiModelRel : model) {
            if (StringUtils.isNotEmpty(ocpApiModelRel.getModelCode())) {
                List<Model> ocpModelBase = ocpBaseMapper.selectModelBase(ocpApiModelRel.getModelCode());
                if (ocpModelBase.size() >= 1) {
                    for (Model model1 : ocpModelBase) {
                        listModel.add(model1);

                        list1Model.add(model1.getApiUnit());
                    }
                    apiRequirementDto.setApiModelNames(listModel);
                    apiRequirementDto.setModelDescription(list1Model);
                }
            }
        }


        //根据apicode查询应用关系表
        List<OcpApiTenantRel> ocpApiTenantRelList = ocptenantRelMapper.selectTenant(ocpApiRequirement.getApiCode());
        for (OcpApiTenantRel ocpApiTenantRel : ocpApiTenantRelList) {
            //根据tenant_code查询每个应用名称
            OcpTenantBase ocpTenantBase = ocpTenantBaseMapper.selectTenants(ocpApiTenantRel.getTenantCode());
            listTenant.add(ocpTenantBase.getTenantDescription());
            list1Tenat.add(ocpTenantBase.getTenantName());
        }
        apiRequirementDto.setTenantDescription(listTenant);
        apiRequirementDto.setApiTenantGroupNames(list1Tenat);

        //查询服务详情的出入参数
        List<OcpApiParam> selects = ocpApiPMapper.selects(ocpApiRequirement.getApiCode());
        BeanUtils.copyProperties(ocpApiRequirement, apiRequirementDto);
        for (OcpApiParam select : selects) {
            if (select.getApiParamClassification().equals("out")) {
                list.add(select);

            } else if (select.getApiParamClassification().equals("in")) {
                lists.add(select);
            }

        }
        apiRequirementDto.setOutList(list);
        apiRequirementDto.setInList(lists);

        return apiRequirementDto;
    }


    /**
     * 服务下线
     * 1
     *
     * @param apiCode
     */
    @Override
    public void updateTables(String apiCode) {
        OcpApiRequirement ocpApiRequirement = ocpApiRequirementMapper.selectByCodes(apiCode);
        List<OcpApiRequirementApproval> requirementApprovals = ocpApiRequirementApprovalMapper.selectByCodes(apiCode);
        if (!(org.springframework.util.StringUtils.isEmpty(ocpApiRequirement))) {
            ocpApiRequirement.setRequirementType((long) 2);
            ocpApiRequirementMapper.updateById(ocpApiRequirement);
//            if (org.springframework.util.StringUtils.isEmpty(requirementApprovals)) {
            for (OcpApiRequirementApproval requirementApproval : requirementApprovals) {
                requirementApproval.setApprovalStats((long) 0);
                requirementApproval.setLastApprovalStats((long) 0);
                ocpApiRequirementApprovalMapper.updateById(requirementApproval);
//            }
            }
        } else {
            ocpApiRequirement.setRequirementType((long) 2);
            ocpApiRequirementMapper.insert(ocpApiRequirement);
            if (org.springframework.util.StringUtils.isEmpty(requirementApprovals)) {
                for (OcpApiRequirementApproval requirementApproval : requirementApprovals) {
                    requirementApproval.setApprovalStats((long) 0);
                    requirementApproval.setLastApprovalStats((long) 0);
                    ocpApiRequirementApprovalMapper.insert(requirementApproval);
                }
            }
        }

    }


    /**
     * 迭代-模型回显
     *
     * @param apiCode
     * @return
     * @Author:郭凡凡
     */
    @Override
    public ResponseVO<List<OcpApiModelRel>> echoModel(String apiCode) {
        LambdaQueryWrapper<OcpApiModelRel> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotEmpty(apiCode), OcpApiModelRel::getApiCode, apiCode);
        List<OcpApiModelRel> list = ocpApiModelRelMapper.selectList(queryWrapper);
        return ResponseVO.ok(list);
    }

    /**
     * @param apiCode
     * @return
     * @Author:郭凡凡 迭代-表字段回显
     */
    @Override
    public ResponseVO<Map<String, List<ApiParam>>> echoTableField(String apiCode) {
        if (StringUtils.isEmpty(apiCode)) {
            return ResponseVO.error("服务编码不能为空");
        }
        Map<String, List<ApiParam>> map = new HashMap<>();
        LambdaQueryWrapper<ApiParam> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(StringUtils.isNotEmpty(apiCode), ApiParam::getApiCode, apiCode);
        wrapper.eq(ApiParam::getProp, "0");
        List<ApiParam> apiParams = paramMapper.selectList(wrapper);
        LambdaQueryWrapper<ApiParam> wrapper1 = new LambdaQueryWrapper<>();
        wrapper1.eq(StringUtils.isNotEmpty(apiCode), ApiParam::getApiCode, apiCode);
        wrapper1.eq(ApiParam::getProp, "1");
        List<ApiParam> apiParam = paramMapper.selectList(wrapper1);
        List<ApiParam> build1 = this.build(apiParam);
        List<ApiParam> build = this.build(apiParams);
        map.put("0", build);
        map.put("1", build1);
        return ResponseVO.ok(map);
    }

    /**
     * 迭代-应用回显
     *
     * @param apiCode
     * @return
     * @Author:郭凡凡
     */
    @Override
    public ResponseVO<List<TenantApiDto>> echoTenant(String apiCode) {
        if (StringUtils.isEmpty(apiCode)) {
            return ResponseVO.error("服务编码不能为空");
        }
        List<TenantApiDto> list = new ArrayList<>();
        LambdaQueryWrapper<OcpApiBase> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotEmpty(apiCode), OcpApiBase::getApiCode, apiCode);
        OcpApiBase ocpApiBase = apiBasicManagementMapper.selectOne(queryWrapper);
        if (ocpApiBase != null) {
            String apiGroupCode = ocpApiBase.getApiGroupCode();
            LambdaQueryWrapper<OcpApiGroup> objectLambdaQueryWrapper = new LambdaQueryWrapper<>();
            objectLambdaQueryWrapper.eq(StringUtils.isNotEmpty(apiGroupCode), OcpApiGroup::getApiGroupCode, apiGroupCode);
            OcpApiGroup ocpApiGroup = ocpApiGroupMapper.selectOne(objectLambdaQueryWrapper);
            String apiGroupName = ocpApiGroup.getApiGroupName();
            String apiGroupPname = ocpApiGroup.getApiGroupPname();
            LambdaQueryWrapper<OcpApiTenantRel> wrapper = new LambdaQueryWrapper<>();
            wrapper.eq(OcpApiTenantRel::getApiCode, apiCode);
            List<OcpApiTenantRel> ocpApiTenantRels = ocpApiTenantRelMapper.selectList(wrapper);
            if (CollectionUtil.isNotEmpty(ocpApiTenantRels)) {
                for (OcpApiTenantRel ocpApiTenantRel : ocpApiTenantRels) {
                    String tenantCode = ocpApiTenantRel.getTenantCode();
                    LambdaQueryWrapper<OcpTenantBase> lambdaQueryWrapper = new LambdaQueryWrapper<>();
                    lambdaQueryWrapper.eq(OcpTenantBase::getTenantCode, tenantCode);
                    OcpTenantBase one = tenantBasicManagementService.getOne(lambdaQueryWrapper);
                    String tenantName = one.getTenantName();
                    Short tenantZone = one.getTenantZone();
                    TenantApiDto tenantApiDto = new TenantApiDto();
                    tenantApiDto.setApiGroupName(apiGroupName);
                    tenantApiDto.setApiGroupPname(apiGroupPname);
                    tenantApiDto.setTenantCode(tenantCode);
                    tenantApiDto.setTenantName(tenantName);
                    tenantApiDto.setApiTenantDescription(ocpApiTenantRel.getApiTenantDescription());
                    tenantApiDto.setTenantZone(tenantZone);
                    list.add(tenantApiDto);
                }
            }
        }
        return ResponseVO.ok(list);
    }

    /**
     * 迭代-服务信息填写
     *
     * @param ocpApiRequirementDto
     * @return
     * @Author:郭凡凡
     */
    @Override
    @Transactional
    public ResponseVO
    updateDemand(OcpApiRequirementDto ocpApiRequirementDto) {

        if (StringUtils.isEmpty(ocpApiRequirementDto.getApiGroupCode())) {
            return ResponseVO.error("服务组编码不能为空");
        }
        if (StringUtils.isEmpty(ocpApiRequirementDto.getApiName())) {
            return ResponseVO.error("服务名称不能为空");
        }
        if (StringUtils.isEmpty(ocpApiRequirementDto.getRequirementComments())) {
            return ResponseVO.error("需求简述不能为空");
        }
        if (StringUtils.isEmpty(ocpApiRequirementDto.getApiVersion())) {
            return ResponseVO.error("服务版本号不能为空");
        }
        if (StringUtils.isEmpty(ocpApiRequirementDto.getApiCode())) {
            return ResponseVO.error("服务编码不能为空");
        }
        if (ocpApiRequirementDto.getRequirementType() == null) {
            return ResponseVO.error("需求类型不能为空");
        }


        //先删除  在添加
        //服务需求表
        LambdaQueryWrapper<OcpApiRequirement> wrapper1 = new LambdaQueryWrapper<>();
        wrapper1.eq(OcpApiRequirement::getApiCode, ocpApiRequirementDto.getApiCode());
        ocpApiRequirementMapper.delete(wrapper1);
        //服务基础表
        LambdaQueryWrapper<OcpApiBase> queryWrapper1 = new LambdaQueryWrapper<>();
        queryWrapper1.eq(OcpApiBase::getApiCode, ocpApiRequirementDto.getApiCode());
        OcpApiBase ocpApiBase1 = apiBasicManagementMapper.selectOne(queryWrapper1);
        Timestamp apiCtime = ocpApiBase1.getApiCtime();
        apiBasicManagementMapper.delete(queryWrapper1);
        //服务扩展表
        LambdaQueryWrapper<OcpApiExtent> wrapper2 = new LambdaQueryWrapper<>();
        wrapper2.eq(OcpApiExtent::getApiCode, ocpApiRequirementDto.getApiCode());
        ocpApiExtentMapper.delete(wrapper2);

        //基础表
        OcpApiBase ocpApiBase = new OcpApiBase();
        ocpApiBase.setIsDelete(0);
        ocpApiBase.setApiCtime(apiCtime);
        ocpApiBase.setApiMtime(Timestamp.valueOf(LocalDateTime.now()));
        ocpApiBase.setApiCode(ocpApiRequirementDto.getApiCode());
        ocpApiBase.setApiName(ocpApiRequirementDto.getApiName());
        ocpApiBase.setApiGroupCode(ocpApiRequirementDto.getApiGroupCode());
        if (ocpApiRequirementDto.getApiPromotion() != null) {
            ocpApiBase.setApiPromotion(ocpApiRequirementDto.getApiPromotion());
        }
        if (StringUtils.isNotEmpty(ocpApiRequirementDto.getApiUnit())) {
            ocpApiBase.setApiUnit(ocpApiRequirementDto.getApiUnit());
        }
        if (StringUtils.isNotEmpty(ocpApiRequirementDto.getApiUrl())) {
            ocpApiBase.setApiUrl(ocpApiRequirementDto.getApiUrl());
        } else {
            ocpApiBase.setApiUrl("/api-control/service");
        }

        //需求表
        OcpApiRequirement ocpApiRequirement = new OcpApiRequirement();
        ocpApiRequirement.setRequirementCode(ocpApiRequirementDto.getRequirementCode());
        ocpApiRequirement.setRequirementType(ocpApiRequirementDto.getRequirementType());
        ocpApiRequirement.setApiGroupCode(ocpApiRequirementDto.getApiGroupCode());
        ocpApiRequirement.setRequirementComments(ocpApiRequirementDto.getRequirementComments());
        ocpApiRequirement.setApiCode(ocpApiRequirementDto.getApiCode());
        ocpApiRequirement.setApiName(ocpApiRequirementDto.getApiName());
        ocpApiRequirement.setApiUnit(ocpApiRequirementDto.getApiUnit());
        ocpApiRequirement.setRequirementUserDepg(ocpApiRequirementDto.getRequirementUserDepg());
        if (ocpApiRequirementDto.getApiPromotion() != null) {
            ocpApiRequirement.setApiPromotion(ocpApiRequirementDto.getApiPromotion());
        }
        if (StringUtils.isNotEmpty(ocpApiRequirementDto.getApiVersion())) {
            ocpApiRequirement.setApiVersion(ocpApiRequirementDto.getApiVersion());
        }
        if (StringUtils.isNotEmpty(ocpApiRequirementDto.getApiUrl())) {
            ocpApiRequirement.setApiUrl(ocpApiRequirementDto.getApiUrl());
        } else {
            ocpApiRequirement.setApiUrl("/api-control/service");
        }
        if (StringUtils.isNotEmpty(ocpApiRequirementDto.getRequirementUserId())) {
            ocpApiRequirement.setRequirementUserName(ocpApiRequirementDto.getRequirementUserId());
        } else {
            ocpApiRequirement.setRequirementUserId("0");
        }
        if (StringUtils.isNotEmpty(ocpApiRequirementDto.getRequirementUserName())) {
            ocpApiRequirement.setRequirementUserName(ocpApiRequirementDto.getRequirementUserName());
        } else {
            ocpApiRequirement.setRequirementUserName("创建者");
        }
        ocpApiRequirement.setRequirementCtime(apiCtime);
        ocpApiRequirement.setRequirementMtime(Timestamp.valueOf(LocalDateTime.now()));
        ocpApiRequirement.setIsDelete((long) 0);


        //获取服务编码，名称 重复校验
        LambdaQueryWrapper<OcpApiBase> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotEmpty(ocpApiRequirementDto.getApiName()), OcpApiBase::getApiName, ocpApiRequirementDto.getApiName());
        queryWrapper.eq(OcpApiBase::getIsDelete, 0);
        List<OcpApiBase> ocpApiBases = apiBasicManagementMapper.selectList(queryWrapper);
        if (ocpApiBases.size() != 0) {
            return ResponseVO.error("服务名称已存在");
        }

        LambdaQueryWrapper<OcpApiBase> queryWrapper3 = new LambdaQueryWrapper<>();
        queryWrapper3.eq(StringUtils.isNotEmpty(ocpApiRequirementDto.getApiCode()), OcpApiBase::getApiCode, ocpApiRequirementDto.getApiCode());
        queryWrapper3.eq(OcpApiBase::getIsDelete, 0);
        List<OcpApiBase> ocpApiBases1 = apiBasicManagementMapper.selectList(queryWrapper3);
        if (ocpApiBases1.size() != 0) {
            return ResponseVO.error("服务编码已存在");
        }


        //也是重复校验 //需求表
        LambdaQueryWrapper<OcpApiRequirement> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(StringUtils.isNotEmpty(ocpApiRequirementDto.getRequirementCode()), OcpApiRequirement::getRequirementCode, ocpApiRequirementDto.getRequirementCode());
        List<OcpApiRequirement> ocpApiRequirements = ocpApiRequirementMapper.selectList(wrapper);
        if (ocpApiRequirements.size() != 0) {
            return ResponseVO.error("需求编码重复");
        }

        //添加到服务需求表 和 服务基础信息表 中
        //添加到服务扩展表中
        OcpApiExtent ocpApiExtent = new OcpApiExtent();
        ocpApiExtent.setApiCode(ocpApiRequirementDto.getApiCode());
        ocpApiExtent.setIsDelete(0);
        ocpApiExtent.setApiCtime(apiCtime);
        ocpApiExtent.setApiMtime(Timestamp.valueOf(LocalDateTime.now()));
        if (StringUtils.isNotEmpty(ocpApiRequirementDto.getClusterName())) {
            ocpApiExtent.setClusterName(ocpApiRequirementDto.getClusterName());
        } else {
            ocpApiExtent.setClusterName("测试集群");
        }
        if (StringUtils.isNotEmpty(ocpApiRequirementDto.getSpaceName())) {
            ocpApiExtent.setSpaceName(ocpApiRequirementDto.getSpaceName());
        } else {
            ocpApiExtent.setSpaceName("开发者");
        }

        int insert1 = ocpApiExtentMapper.insert(ocpApiExtent);
        int insert = apiBasicManagementMapper.insert(ocpApiBase);
        int insert2 = ocpApiRequirementMapper.insert(ocpApiRequirement);
        if ((insert2 > 0) && (insert > 0) && (insert1 > 0)) {
            return ResponseVO.ok();
        }
        return ResponseVO.error("添加失败");
    }


    /**
     * 迭代-模型入参出参回显
     *
     * @param apiCode
     * @return
     * @Author:郭凡凡
     */
    @Override
    public ResponseVO<List<OcpApiParam>> echoModelParameter(String apiCode) {
        LambdaQueryWrapper<OcpApiParam> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(StringUtils.isNotEmpty(apiCode), OcpApiParam::getApiCode, apiCode);
        List<OcpApiParam> list = ocpApiPMapper.selectList(wrapper);
        return ResponseVO.ok(list);
    }

    /**
     * 审核状态
     *
     * @param apiCode
     * @return
     * @Author:郭凡凡
     */
    @Override
    public ResponseVO selectAuditStatus(String apiCode) {
        LambdaQueryWrapper<OcpApiRequirementApproval> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(StringUtils.isNotEmpty(apiCode), OcpApiRequirementApproval::getApiCode, apiCode);

        List<OcpApiRequirementApproval> list = ocpApiRequirementApprovalMapper.selectList(wrapper);
        for (OcpApiRequirementApproval approval : list) {
            if (approval.getApprovalStats() == 0) {
                return ResponseVO.error("未审核,请审核后在迭代");
            }
            if (approval.getApprovalStats() == 2) {
                return ResponseVO.error("审核拒绝");
            }
        }
        return ResponseVO.ok();
    }


    //build
    private List<ApiParam> build(List<ApiParam> treeNodes) {
        List<ApiParam> result = new ArrayList<>();
        //list转map =>将每个对象的id作为key,自己作为value.
        Map<String, ApiParam> nodeMap = new LinkedHashMap<>(treeNodes.size());
        for (ApiParam treeNode : treeNodes) {
            nodeMap.put(treeNode.getObjId(), treeNode);
        }
        for (ApiParam node : nodeMap.values()) {
            //得到自己的父类
            ApiParam parent = nodeMap.get(node.getPid());
            if (parent != null && (node.getPid().equals(parent.getObjId()))
                    && (!"".equals(node.getPid()) || !"".equals(parent.getObjId()))
                    && !parent.getObjId().equals(node.getObjId())) {
                //防止每次都new list或者只是使用该对象就new对象浪费内存或者覆盖上一次记录
                if (parent.getChildrenList() == null) {
                    parent.setChildrenList(new LinkedList<>());
                }
                //设置上级父类名称
                parent.getChildrenList().add(node);
                continue;
            }
            result.add(node);
        }
        return result;
    }

}


